#!/bin/sh
##
## Advanced network settings for UR5i-v2 router
## Version 0.0.1b   4/15/2014
##

MODULE_NAME="advanced_security"
PREFIX="MOD_SECURITY_"
SETTINGS_FILE="/opt/$MODULE_NAME/etc/settings"
ACCESS_LIST="/opt/$MODULE_NAME/etc/access_list"
LAST_REPORT_LOCATION="/var/$MODULE_NAME/"
LAST_REPORT_FILE="$LAST_REPORT_LOCATION/last_report"
SSHD_CONFIG="/etc/sshd.conf"
SECURITY_INPUT_CHAIN_NAME="in_security"
SECURITY_OUTPUT_CHAIN_NAME="out_security"
SPACE_PATTERN="[[:space:]]\{1,\}"
BANNER_PATTERN="^[[:space:]]*Banner[[:space:]]/opt/$MODULE_NAME/etc/welcome.msg"



#======================= START OF HELPER FUNCTIONS =====================#


execute_iptables_command ()
{
  eval iptables" $1" > /dev/null 2>&1
  if [ "$?" = "0" ] ; then
    return 0
  else
    echo "IPTABLES command failed: $1"
    return 1
  fi
}

is_iptable_rule_enabled ()
{
  INPUT_CHAIN_LIST=$(iptables -L "$SECURITY_INPUT_CHAIN_NAME"  |sed '1d;2d' )
  INPUT_STRING=$(echo "$1"  |  sed "s/[[:space:]]\+/\[\[\:space\:\]\]\\\{1,\\\}/g")
  POSITION=$(echo "$INPUT_CHAIN_LIST" | grep -n  "$INPUT_STRING" |awk -F ":" '{print $1}')

  if [ "$?" = "0" ] ; then
    # Is it realy a number ?
    echo "$POSITION" | grep -qe  "^[0-9]\+$"
    if [ "$?" = "0" ] ; then
      #echo "rule enabled $POSITION"
      return 0
    fi
  fi
  #echo "rule not enabled"
  return 1

}

delete_iptable_rule ()
{
  INPUT_CHAIN_LIST=$(iptables -L "$SECURITY_INPUT_CHAIN_NAME"  |sed '1d;2d' )
  INPUT_STRING=$(echo "$1"  |  sed "s/[[:space:]]\+/\[\[\:space\:\]\]\\\{1,\\\}/g")
  POSITION=$(echo "$INPUT_CHAIN_LIST" | grep -n  "$INPUT_STRING" |awk -F ":" '{print $1}')

  if [ "$?" = "0" ] ; then
    # Is it realy a number ?
    echo "$POSITION" | grep -qe  "^[0-9]\+$"
    if [ "$?" = "0" ] ; then
      execute_iptables_command "-D $SECURITY_INPUT_CHAIN_NAME $POSITION"
      return 0
    fi
  fi
  return 1
}


delete_iptable_rule_if_enabled ()
{
  is_iptable_rule_enabled   "$1"
  if [ "$?" = "0" ] ; then
    delete_iptable_rule "$1"
  fi
}

exec_iptable_rule_if_not_present ()
{
  is_iptable_rule_enabled  "$1"
  if [ "$?" = "1" ] ;then
    execute_input_chain_command "$2"
    return 0
  else
    return 1
  fi
}


execute_input_chain_command ()
{
  eval iptables" -A $SECURITY_INPUT_CHAIN_NAME $1" > /dev/null 2>&1
  if [ "$?" = "0" ] ; then
    return 0
  else
    echo "IPTABLES command failed: $1"
    return 1
  fi
}

execute_output_chain_command ()
{
  eval iptables" -A $SECURITY_OUTPUT_CHAIN_NAME $1" > /dev/null 2>&1
  if [ "$?" = "0" ] ; then
    return 0
  else
    echo "IPTABLES command failed: $1"
    return 1
  fi
}




get_network_base_addr ()
{
  IP=$(ifconfig $1 | awk -F"[ :]+" '/inet addr/ && !/127.0/ {print $4}')
  NET_MASK=$(ifconfig $1 | sed -rn '2s/ .*:(.*)$/\1/p')
  NETWORK_BASE_ADDR=$(echo "$IP"".""$NET_MASK" | { IFS=. read -r i1 i2 i3 i4 m1 m2 m3 m4; echo "$((i1 & m1)).$(($i2 & m2)).$((i3 & m3)).$((i4 & m4))" ;})
  is_valid_ip  "$NETWORK_BASE_ADDR"

  if [ "$?" = 0 ] ; then
    echo  "$NETWORK_BASE_ADDR"
  fi
}

get_broadcast_addr ()
{
  BROADCAST_ADDR=$(ifconfig $1 |grep -v 127.0.0.1 | grep inet | awk '{print $3}'|awk -F ":" '{print $2}' | sed /^$/d)
  is_valid_ip  "$BROADCAST_ADDR"
  if [ "$?" = 0 ] ; then
    echo "$BROADCAST_ADDR"
  fi
}


is_valid_ip ()
{
  case "$*" in
    ""|*[!0-9.]*|*[!0-9]) return 1 ;;
  esac

  IFS=.  ## local is bash-specific
  set -- $*
  [ $# -eq 4 ] &&
  [ ${1:-666} -le 255 ] && [ ${2:-666} -le 255 ] &&
  [ ${3:-666} -le 255 ] && [ ${4:-666} -le 255 ]   # Warning 255 is also valid !!!
}

get_settings_value ()
{
  VALUE=$(grep "$PREFIX$1" "$SETTINGS_FILE" | awk -F "=" '{print $2}')

  if [ "$?" = "0" ] ; then
    # Is it realy a number ?
    echo "$VALUE" | grep -qe  "^[0-9]\+$"
    if [ "$?" = "0" ] ; then
      echo "$VALUE"
      return 0
    fi
  fi
  echo " "
  return 1

}


set_one_if_zero ()
{

  grep  -qe "0" "$1"
  if [ "$?" = "0" ] ; then
    echo "1" > "$1"
    if [ "$?" = "1" ] ; then
      echo "Error: could not set $1 to 0."
      return 1
    fi
  fi
  return 0
}

set_zero_if_one ()
{
  grep  -qe "1" "$1"
  if [ "$?" = "0" ] ; then
    echo "0" > "$1"
    if [ "$?" = "1" ] ; then
      echo "Error: could not set $1 to 0."
      return 1
    fi
  fi
  return 0
}



generate_report ()
{

  INTERFACES=$(ip link show up |grep "^[0-9]:" |grep -v "lo"| awk -F ":" '{printf $2}' | sed -e 's/^[ \t]*//')

  for interface in `echo $INTERFACES`
  do

    IP=$(ifconfig $interface | awk -F"[ :]+" '/inet addr/ && !/127.0/ {print $4}')
    NET_MASK=$(ifconfig $interface | sed -rn '2s/ .*:(.*)$/\1/p')

    echo "$interface;$IP"
    echo "$interface;$NET_MASK"
  done

}

compare_report ()
{
  OLD_REPORT=$(cat $LAST_REPORT_FILE)
  NEW_REPORT=$(generate_report)

  if [ "$OLD_REPORT" != "$NEW_REPORT" ] ; then
    return 1
  else
    echo "OLD_REPORT=$OLD_REPORT"
    echo "NEW_REPORT=$NEW_REPORT"
    return 0
  fi
}




remove_duplicate_from_list ()
{
  # Unfortunately we dont have sort or uniq so we have go with dirty way
  LIST="$1"
  DUPLICATE_LIST=""
  position=0
  for what in `echo $LIST` ; do


    position_b=0
    position_a=$((position_a+1))

    for item in `echo $LIST` ; do
      position_b=$((position_b+1))

      if [ "$item" == "$what" ] && [ $position_b -ne $position_a ] ; then
        DUPLICATE_LIST=$DUPLICATE_LIST$what" "
        #echo "same "
        NEW_LIST=$(echo "$LIST" | sed "s/$what/\"\"/g")
          #echo "NL $NEW_LIST"
          LIST=$NEW_LIST
          break
        fi
      done

    done


    echo "$DUPLICATE_LIST$LIST" | sed 's/\"//g' | sed  's/[[:space:]]\{1,\}/ /g'
  }


  #======================= END OF HELPER FUNCTIONS =======================#

  enable_sshd_banner () {
    grep -qe "$BANNER_PATTERN" "$SSHD_CONFIG"
    if [ "$?" = "0" ] ; then
      echo "Already has enabled SSHD banner, skipping."
    else

      sed -i -e '$aBanner /opt/'"$MODULE_NAME"'/etc/welcome.msg' "$SSHD_CONFIG"

      echo "SSHD restart needed."
      /etc/init.d/ssh stop
      /etc/init.d/ssh start
      echo "SSHD banner enabled."
    fi
  }

  disable_sshd_banner () {
    grep -qe "$BANNER_PATTERN" "$SSHD_CONFIG"
    if [ "$?" = "0" ] ; then
      sed -i -e '/Banner \/opt\/'"$MODULE_NAME"'\/etc\/welcome.msg/d' "$SSHD_CONFIG"
      echo "SSHD restart needed."
      /etc/init.d/ssh stop
      /etc/init.d/ssh start
      echo "SSHD banner disabled."
    else
      echo "SSHD banner not enabled, skipping."
    fi
  }


  is_host_already_enabled_for_ssh ()
  {
    grep -qse "$1"  "$ACCESS_LIST"
    if [ "$?" = "0" ] ; then
      return 0
    else
      return 1
    fi
  }

  block_all_sshd_hosts ()
  {
    echo "Block access to ssh for all unlisted hosts."
    exec_iptable_rule_if_not_present "DROP tcp -- anywhere anywhere tcp dpt:ssh"  " -p tcp --dport 22 -j DROP"

  }

  block_all_telnet_hosts ()
  {
    echo "Block access to telnet for all unlisted hosts."
    exec_iptable_rule_if_not_present "DROP tcp -- anywhere anywhere tcp dpt:telnet"  " -p tcp --dport 23 -j DROP"

  }



  enable_sshd_host ()
  {

    is_valid_ip "$1"

    if [ "$?" = "0" ] ; then
      is_host_already_enabled_for_ssh  "$1"
      if [ "$?" = "1" ] ; then
        if [ -r "$ACCESS_LIST" ]	; then
          sed -i -e '$a '"$1" "$ACCESS_LIST"
        else
          echo "$1" >> "$ACCESS_LIST"
        fi
      fi
    else
      echo "Not valid IP"
    fi

  }


  disable_sshd_host ()
  {
    echo "disable_sshd_host"
    is_valid_ip "$1"

    if [ "$?" = "0" ] ; then
      is_host_already_enabled_for_ssh  "$1"

      if [ "$?" = "0" ] ; then
        echo "disabling"
        if [ -r "$ACCESS_LIST" ]	; then
          sed -i "/$1/d" "$ACCESS_LIST"
        else
          echo "Access list empty, disabling host not needed"
        fi
      fi
    else
      echo "Not valid IP, could not disable ssh access for it"
    fi
  }



  process_ssh_acess_list ()
  {

    echo "Restricting SSHD acess"
    while read line
    do
      ip_address="$line"
      is_valid_ip  "$ip_address"

      if [ "$?" = 0 ] ; then
        echo "SSH acess allowed for $ip_address"
        exec_iptable_rule_if_not_present "ACCEPT tcp -- $ip_address anywhere tcp dpt:ssh" " -p tcp -s $ip_address --dport 22 -j ACCEPT"
      fi
    done < "$ACCESS_LIST"
    block_all_sshd_hosts

  }


  process_telnet_acess_list ()
  {

    echo "Restricting Telnet acess"
    while read line
    do
      ip_address="$line"
      is_valid_ip  "$ip_address"

      if [ "$?" = 0 ] ; then
        echo "SSH acess allowed for $ip_address"
        exec_iptable_rule_if_not_present "ACCEPT tcp -- $ip_address anywhere tcp dpt:telnet" " -p tcp -s $ip_address --dport 23 -j ACCEPT"
      fi
    done < "$ACCESS_LIST"
    block_all_telnet_hosts

  }


  # https://www.gentoo.org/doc/en/security/security-handbook.xml?part=1&chap=9

  disable_source_routing ()
  {
    set_zero_if_one "/proc/sys/net/ipv4/conf/all/accept_source_route"
    set_zero_if_one "/proc/sys/net/ipv4/conf/default/accept_source_route"
  }

  enable_source_routing ()
  {
    set_one_if_zero "/proc/sys/net/ipv4/conf/all/accept_source_route"
    set_one_if_zero "/proc/sys/net/ipv4/conf/default/accept_source_route"
  }


  #http://darkness.codefu.org/wordpress/2006/02/linux-and-directed-broadcasts/
  #http://www.mycert.org.my/en/resources/network/main/main/detail/537/index.html


  disable_directed_broadcast ()
  {

    INTERFACES=$(ip link show up |grep "^[0-9]:" |grep -v "lo"| awk -F ":" '{printf $2}' | sed -e 's/^[ \t]*//')





    for interface in `echo $INTERFACES`
    do

      # following is consistent with  RFC 1122

      NET_BASE=$(get_network_base_addr $interface)
      BROADCAST_ADDR=$(get_broadcast_addr $interface)



      if [ -n "$NET_BASE"  ] || [ -n "$BROADCAST_ADDR" ] ; then

        exec_iptable_rule_if_not_present "DROP icmp -- anywhere $BROADCAST_ADDR"  "-p icmp -d $BROADCAST_ADDR -j DROP"
        exec_iptable_rule_if_not_present "DROP icmp -- anywhere $NET_BASE"  "-p icmp -d $NET_BASE -j DROP"
      else

        echo "No IP on interface $interface"

      fi

    done


    # following is more restrictive and not consistent with  RFC 1122
    #set_one_if_zero  /proc/sys/net/ipv4/icmp_echo_ignore_broadcasts


  }


  enable_directed_broadcast ()
  {

    NET_BASE=$(get_network_base_addr eth0)
    BROADCAST_ADDR=$(get_broadcast_addr eth0)

    delete_iptable_rule_if_enabled "DROP all -- anywhere $BROADCAST_ADDR"
    delete_iptable_rule_if_enabled "DROP all --  anywhere  $NET_BASE"

    # following is more restrictive and not consistent with  RFC 1122
    #set_zero_if_one /proc/sys/net/ipv4/icmp_echo_ignore_broadcasts


  }


  disable_ip_classless_routing ()
  {
    echo "==== disable_ip_classless_routing  === "

    # OUTPUT FORWARD ?

    ROUTES=$(ip route list| awk '{print $1}' |grep -v "default")


    CLASSB_LIST=""
    for route_address in `echo $ROUTES`
    do
      execute_output_chain_command "-d $route_address -j ACCEPT"
      echo "iptables -A $SECURITY_OUTPUT_CHAIN_NAME -d $route_address -j ACCEPT"
      CLASSB=$(echo "$route_address" |  { IFS=. read -r i1 i2 i3 i4 ; echo "$i1.$i2.0.0";})
      TMP_LIST="$CLASSB_LIST $CLASSB"
      CLASSB_LIST="$TMP_LIST"
    done

    TMP=$(remove_duplicate_from_list  `echo $CLASSB_LIST`)

    for route_address in `echo $TMP `
    do
      execute_output_chain_command "-d $route_address -j DROP"
      echo "iptables -A $SECURITY_OUTPUT_CHAIN_NAME -d $route_address -j DROP"
    done

    echo "==== disable_ip_classless_routing  === "

  }

  enable_ip_classless_routing ()
  {
    echo "enable_ip_classless_routing , not implemented"
  }


  #http://www.oregontechsupport.com/articles/icmp.txt
  #http://www.faqs.org/docs/iptables/icmptypes.html

  disable_icmp_messages ()
  {
    execute_input_chain_command  "-p icmp --icmp-type 0 -j ACCEPT"
    execute_output_chain_command "-p icmp --icmp-type 0 -j ACCEPT"

    # ICMP types 1-2 are unassigned

    # ICMP destination-unreachable (type 3)

    # Filtering "fragmentation-needed" traffic is a BAD idea.
    execute_input_chain_command "-p icmp --icmp-type 3/4 -j ACCEPT"
    execute_output_chain_command "-p icmp --icmp-type 3/4 -j ACCEPT"

    # Port unreachable
    #execute_input_chain_command "-p icmp --icmp-type 3/3 -j ACCEPT"
    #execute_output_chain_command "-p icmp --icmp-type 3/3 -j ACCEPT"

    # Host unreachable

    #execute_input_chain_command "-p icmp --icmp-type 3/1 -j ACCEPT"
    #execute_output_chain_command "-p icmp --icmp-type 3/1 -j ACCEPT"

    # ICMP Source Quench (type 4)
    execute_input_chain_command "-p icmp --icmp-type 4 -j ACCEPT"
    execute_output_chain_command "-p icmp --icmp-type 4 -j ACCEPT"

    # ICMP Redirect (type 5)
    set_zero_if_one "/proc/sys/net/ipv4/conf/all/accept_redirects"
    # ICMP Alternate Host Address (type 6)

    # ICMP Unassigned (type 7)

    # ICMP Echo Request (type 8)

    execute_input_chain_command "-p icmp --icmp-type 8 -m limit --limit 2/s -j ACCEPT"
    execute_input_chain_command "-p icmp --icmp-type 8 -j DROP"
    execute_output_chain_command "-p icmp --icmp-type 8 -j ACCEPT"

    # ICMP Router Advertisement (type 9)
    # ICMP Router Solicitation (type 10)
    # ICMP Time Exceeded (type 11)
    execute_input_chain_command "-p icmp --icmp-type 11 -j ACCEPT"
    execute_output_chain_command "-p icmp --icmp-type 11 -j ACCEPT"
    # ICMP Parameter Problem (type 12)
    execute_input_chain_command "-p icmp --icmp-type 12 -j ACCEPT"
    execute_output_chain_command "-p icmp --icmp-type 12 -j ACCEPT"

    # ICMP Timestamp (type 13)
    # ICMP Timestamp Reply (type 14)
    # ICMP Information Request (type 15)
    # ICMP Information Reply (type 16)
    # ICMP Address Mask Request (type 17)
    # ICMP Address Mask Reply (type 18)
    # ICMP Reserved (types 19-29)
    # ICMP Traceroute (type 30)
    # ICMP Datagram Conversion Error (type 31)
    set_one_if_zero "/proc/sys/net/ipv4/icmp_ignore_bogus_error_responses"
    # ICMP Catch-all remaining
    execute_input_chain_command "-p icmp -j DROP"
    execute_output_chain_command "-p icmp -j DROP"

  }




  enable_icmp_messages ()
  {

    set_zero_if_one "/proc/sys/net/ipv4/icmp_ignore_bogus_error_responses"
    set_one_if_zero "/proc/sys/net/ipv4/conf/all/accept_redirects"

  }



  disable_dns_name_resolution_broadcast ()
  {
    # not interfering with DHCP

    # DROP       udp  --  anywhere             255.255.255.255     udp dpt:domain
    # DROP       tcp  --  anywhere             255.255.255.255     tcp dpt:domain

    # could not confirm this behaviour on Linux
    exec_iptable_rule_if_not_present "DROP       udp  --  anywhere             255.255.255.255     udp dpt:domain"   "-d 255.255.255.255 -p udp --dport 53 -j DROP"
    exec_iptable_rule_if_not_present "DROP       tcp  --  anywhere             255.255.255.255     tcp dpt:domain"   "-d 255.255.255.255 -p tcp --dport 53 -j DROP"

  }


  enable_dns_name_resolution_broadcast ()
  {
    #delete_iptable_rule_if_enabled "DROP all --  anywhere 255.255.255.255"
    delete_iptable_rule_if_enabled "DROP       udp  --  anywhere             255.255.255.255     udp dpt:domain"
    delete_iptable_rule_if_enabled "DROP       tcp  --  anywhere             255.255.255.255     tcp dpt:domain"
  }

  disable_icmp_protocol ()
  {

    delete_iptable_rule_if_enabled "ACCEPT icmp -- anywhere  anywhere"
    exec_iptable_rule_if_not_present  "DROP icmp -- anywhere anywhere"   " -p icmp -j DROP"

    # following is optional

    #set_one_if_zero "/proc/sys/net/ipv4/icmp_echo_ignore_all"


  }

  enable_icmp_protocol ()
  {
    delete_iptable_rule_if_enabled    "DROP icmp  --  anywhere anywhere"

    exec_iptable_rule_if_not_present  "ACCEPT icmp -- anywhere anywhere"  " -p icmp -j ACCEPT"


    # following is optional

    #set_zero_if_one  "/proc/sys/net/ipv4/icmp_echo_ignore_all"

  }


  disable_fragmentation ()
  {
    exec_iptable_rule_if_not_present  "DROP all -f anywhere  anywhere"  " -f -j DROP"
  }

  enable_fragmentation  ()
  {
    delete_iptable_rule_if_enabled "DROP all -f anywhere  anywhere"
  }

  # https://bugzilla.kernel.org/show_bug.cgi?id=18952

  set_wait_for_establish ()
  {

    # (2xIp)+1

    case "$1"  in
      "1")
        VALUE=3
      ;;
      "2")
        VALUE=7
      ;;
      "3")
        VALUE=15
      ;;
      "4")
        VALUE=31
      ;;
      "5")
        VALUE=63
      ;;
      "6")
        VALUE=127
      ;;

    esac

    echo "TCP SYN timeout set to $VALUE seconds"
    echo "$1" > /proc/sys/net/ipv4/tcp_syn_retries

    #iptables "-A INPUT -p tcp -m state --state NEW  -m recent --update --seconds $1 -j DROP"
    #iptables -A INPUT -p tcp -m state --state NEW  -m recent --set -j ACCEPT


    #delete_iptable_rule_if_enabled "DROP tcp -- anywhere  anywhere  state NEW recent: UPDATE seconds: [0-9]\{1,\} name: DEFAULT side: source"
    #execute_chain_command "-p tcp -m state --state NEW  -m recent --update --seconds $1 -j DROP"




  }

  set_ssh_idle_timeout()
  {
    sed -ie "/.*ClientAliveInterval.*/d" $SSHD_CONFIG
    sed -ie "/.*ClientAliveCountMax.*/d" $SSHD_CONFIG

    echo "ClientAliveInterval $1" >> $SSHD_CONFIG
    echo "ClientAliveCountMax 0" >> $SSHD_CONFIG

    echo "SSHD restart needed."
    /etc/init.d/ssh stop
    /etc/init.d/ssh start
  }

  prepare_iptables ()
  {
    iptables -L "$SECURITY_INPUT_CHAIN_NAME" > /dev/null 2>&1

    if  [ "$?" = "1" ] ; then
      execute_iptables_command "-N $SECURITY_INPUT_CHAIN_NAME"
      execute_iptables_command "-t filter -A INPUT -j $SECURITY_INPUT_CHAIN_NAME"
    else
      execute_iptables_command "--flush $SECURITY_INPUT_CHAIN_NAME"
    fi

    iptables -L "$SECURITY_OUTPUT_CHAIN_NAME" > /dev/null 2>&1

    if  [ "$?" = "1" ] ; then
      execute_iptables_command "-N $SECURITY_OUTPUT_CHAIN_NAME"
      execute_iptables_command "-t filter -A OUTPUT -j $SECURITY_OUTPUT_CHAIN_NAME"
    else
      execute_iptables_command "--flush $SECURITY_OUTPUT_CHAIN_NAME"
    fi

  }



  setup ()
  {
    #  This is setup function, which is called by set.cgi after any change
    #  of options

    date

    mkdir  -p /var/advanced_security
    generate_report >  "$LAST_REPORT_FILE"

    prepare_iptables


    SETTINGS_VALUE=$(get_settings_value "SSHD_BANNER")

    if [ "$SETTINGS_VALUE" = "0" ] ; then
      echo "Enabling SSHD banner."
      enable_sshd_banner
    else
      echo "Disabling SSHD banner."
      disable_sshd_banner
    fi



    SETTINGS_VALUE=$(get_settings_value "SOURCE_ROUTING")

    if [ "$SETTINGS_VALUE" = "0" ] ; then
      echo "Enabling source routing."
      enable_source_routing
    else
      echo "Disabling source routing."
      disable_source_routing
    fi



    SETTINGS_VALUE=$(get_settings_value "DIRECTED_BROADCAST")

    if [ "$SETTINGS_VALUE" = "0" ] ; then
      echo "Enabling directed broadcast."
      #enable_directed_broadcast
    else
      echo "Disabling directed broadcast."
      disable_directed_broadcast
    fi


    SETTINGS_VALUE=$(get_settings_value "IP_CLASSLESS_ROUTING")

    if [ "$SETTINGS_VALUE" = "0" ] ; then
      echo "Enabling IP_CLASSLESS_ROUTING."
      enable_ip_classless_routing
    else
      echo "Disabling IP_CLASSLESS_ROUTING."
      disable_ip_classless_routing
    fi


    SETTINGS_VALUE=$(get_settings_value "DNS_NAME_RESOLUTION")

    if [ "$SETTINGS_VALUE" = "0" ] ; then
      echo "Enabling DNS broadcasting."
      #enable_dns_name_resolution_broadcast
    else
      echo "Disabling DNS broadcasting."
      disable_dns_name_resolution_broadcast
    fi


    SETTINGS_VALUE=$(get_settings_value "ICMP_PROTOCOL")

    if [ "$SETTINGS_VALUE" = "0" ] ; then
      echo "Enabling ICMP protocol."
      enable_icmp_protocol

      SETTINGS_VALUE=$(get_settings_value "ICMP_MESSAGES")

      if [ "$SETTINGS_VALUE" = "0" ] ; then
        echo "Enabling ICMP messages."
        enable_icmp_messages
      else
        echo "Disabling ICMP messages."
        disable_icmp_messages
      fi


    else
      echo "Disabling ICMP protocol."
      disable_icmp_protocol
    fi


    SETTINGS_VALUE=$(get_settings_value "FRAGMENTATION")

    if [ "$SETTINGS_VALUE" = "0" ] ; then
      echo "Enabling fragmentation."
      #enable_fragmentation
    else
      echo "Disabling fragmentation."
      disable_fragmentation
    fi

    SETTINGS_VALUE=$(get_settings_value "WAIT_FOR_ESTABLISH")

    if [ -n "$SETTINGS_VALUE"  ] ; then


      set_wait_for_establish "$SETTINGS_VALUE"
    fi


    SETTINGS_VALUE=$(get_settings_value "RESTRICT_SSH_ACCESS")

    if [ "$SETTINGS_VALUE" = "0" ] ; then
      process_ssh_acess_list
    fi


    SETTINGS_VALUE=$(get_settings_value "RESTRICT_TELNET_ACCESS")

    if [ "$SETTINGS_VALUE" = "0" ] ; then
      process_telnet_acess_list
    fi

    SETTINGS_VALUE=$(get_settings_value "SSH_IDLE_TIMEOUT")

    if [ -n "$SETTINGS_VALUE"  ] ; then

      set_ssh_idle_timeout "$SETTINGS_VALUE"
    fi


    date
  }



  if [ "$1" = "setup" ] ; then
    setup
    elif [ "$1" = "enable_sshd_host" ] ; then
    enable_sshd_host $2
    setup
    elif [ "$1" = "disable_sshd_host" ] ; then
    disable_sshd_host $2
    setup
  fi





